package com.sunyard.chsm.sdf;


import com.sunyard.chsm.sdf.model.EccKey;
import com.sunyard.chsm.sdf.model.EccPriKey;
import com.sunyard.chsm.sdf.model.EccPubKey;
import com.sunyard.chsm.utils.gm.BCSM2Utils;
import com.sunyard.chsm.utils.gm.BCSM3Utils;
import lombok.SneakyThrows;
import org.bouncycastle.crypto.digests.SM3Digest;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPrivateKey;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
import org.bouncycastle.util.BigIntegers;
import org.springframework.stereotype.Service;

import java.security.KeyPair;
import java.security.SecureRandom;

/**
 * @author liulu
 * @since 2024/10/23
 */
@Service
public class BCSdfApiService implements SdfApiService {


    @Override
    public byte[] generateRandom(int len) {
        byte[] res = new byte[len];
        new SecureRandom().nextBytes(res);
        return res;
    }

    @SneakyThrows
    @Override
    public EccKey genKeyPairEcc() {
        // 生成密钥对
        KeyPair keyPair = BCSM2Utils.generateKeyPair();
        BCECPublicKey pubKey = (BCECPublicKey) keyPair.getPublic();
        BCECPrivateKey priKey = (BCECPrivateKey) keyPair.getPrivate();
        byte[] x = pubKey.getQ().getXCoord().getEncoded();
        byte[] y = pubKey.getQ().getYCoord().getEncoded();
        byte[] d = BigIntegers.asUnsignedByteArray(32, priKey.getD());

        return new EccKey(new EccPubKey(256, x, y), new EccPriKey(256, d));
    }


    @Override
    public byte[] calculateMAC(byte[] symKey, byte[] pucIv, byte[] pucData) {
        return new byte[0];
    }

    @Override
    public byte[] hmac(byte[] key, byte[] srcData) {
        KeyParameter keyParameter = new KeyParameter(key);
        SM3Digest digest = new SM3Digest();
        HMac mac = new HMac(digest);
        mac.init(keyParameter);
        mac.update(srcData, 0, srcData.length);
        byte[] result = new byte[mac.getMacSize()];
        mac.doFinal(result, 0);
        return result;
    }

    @Override
    public byte[] hash(byte[] pucData) {
        return BCSM3Utils.hash(pucData);
    }

    @Override
    public byte[] encryptByTMK(byte[] data) {
        return data;
    }

    @Override
    public byte[] decryptByTMK(byte[] data) {
        return data;
    }

}
