package com.sunyard.chsm.sdf.adapter;

import com.sun.jna.Pointer;
import com.sun.jna.ptr.PointerByReference;
import com.sunyard.chsm.sdf.lib.SdfLibrary;
import com.sunyard.chsm.sdf.model.DeviceInfo;
import com.sunyard.chsm.sdf.model.EccKey;
import com.sunyard.chsm.sdf.model.EccPubKey;
import lombok.RequiredArgsConstructor;
import org.springframework.util.Assert;

import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author liulu
 * @since 2024/11/4
 */
@RequiredArgsConstructor
public abstract class JnaSdfAdaptor implements SdfApiAdapter {

    protected static final Map<String, Pointer> DEVICE_HANDLE_CONTEXT = new ConcurrentHashMap<>();
    protected static final Map<String, Pointer> SESSION_HANDLE_CONTEXT = new ConcurrentHashMap<>();
    protected static final Map<String, Pointer> KEY_HANDLE_CONTEXT = new ConcurrentHashMap<>();

    protected final SdfLibrary sdfLibrary;

    protected abstract int getAlgId(String alg);

    @Override
    public String openDevice() {
        PointerByReference phDeviceHandle = new PointerByReference();
        sdfLibrary.SDF_OpenDevice(phDeviceHandle);
        String key = UUID.randomUUID().toString();
        DEVICE_HANDLE_CONTEXT.put(key, phDeviceHandle.getValue());
        return key;
    }

    @Override
    public boolean closeDevice(String deviceHandle) {
        Pointer pointer = DEVICE_HANDLE_CONTEXT.remove(deviceHandle);
        if (pointer != null) {
            int res = sdfLibrary.SDF_CloseDevice(pointer);
            return res == 0;
        }
        return true;
    }

    @Override
    public String openSession(String deviceHandle) {
        Pointer device = getDeviceHandle(deviceHandle);

        PointerByReference phSessionHandle = new PointerByReference();
        sdfLibrary.SDF_OpenSession(device, phSessionHandle);

        String key = UUID.randomUUID().toString();
        SESSION_HANDLE_CONTEXT.put(key, phSessionHandle.getValue());
        return key;
    }

    @Override
    public void closeSession(String sessionHandle) {
        Pointer pointer = SESSION_HANDLE_CONTEXT.remove(sessionHandle);
        if (pointer != null) {
            sdfLibrary.SDF_CloseSession(pointer);
        }
    }

    protected Pointer getDeviceHandle(String deviceHandle) {
        Pointer pointer = DEVICE_HANDLE_CONTEXT.get(deviceHandle);
        Assert.notNull(pointer, "设备句柄不存在");
        return pointer;
    }

    protected Pointer getSessionHandle(String sessionHandle) {
        Pointer pointer = SESSION_HANDLE_CONTEXT.get(sessionHandle);
        Assert.notNull(pointer, "会话句柄不存在");
        return pointer;
    }

    protected Pointer getKeyHandle(String keyHandle) {
        Pointer pointer = KEY_HANDLE_CONTEXT.get(keyHandle);
        Assert.notNull(pointer, "密钥句柄不存在");
        return pointer;
    }

    @Override
    public DeviceInfo getDeviceInfo(String sessionHandle) {
        return null;
    }

    @Override
    public byte[] generateRandom(String sessionHandle, int uiLength) {
        byte[] pucRandom = new byte[uiLength];
        Pointer hSessionHandle = getSessionHandle(sessionHandle);
        sdfLibrary.SDF_GenerateRandom(hSessionHandle, uiLength, pucRandom);
        return pucRandom;
    }

    @Override
    public EccPubKey exportEncPublicKeyECC(String sessionHandle, int uiKeyIndex) {
        return null;
    }

    @Override
    public EccKey generateKeyPairECC(String sessionHandle, String alg, int uiKeyBits) {
        return null;
    }
}
