package com.sunyard.chsm.sdf.lib;

import com.sun.jna.Library;
import com.sun.jna.Pointer;
import com.sun.jna.ptr.PointerByReference;

/**
 * @author liulu
 * @since 2024/11/4
 */
public interface SdfLibrary extends Library {

    /**
     * 打开设备
     *
     * @param phDeviceHandle 返回设备句柄
     * @return 0 成功;   非0 失败,返回错误代码
     */
    int SDF_OpenDevice(PointerByReference phDeviceHandle);

    /**
     * 关闭设备
     *
     * @param hDeviceHandle 已打开的设备句柄
     * @return 0 成功;   非0 失败,返回错误代码
     */
    int SDF_CloseDevice(Pointer hDeviceHandle);

    /**
     * 创建会话
     *
     * @param hDeviceHandle   已打开的设备句柄
     * @param phSessionHandle 返回与密码设备建立的新会话句柄
     * @return 0 成功;   非0 失败,返回错误代码
     */
    int SDF_OpenSession(Pointer hDeviceHandle, PointerByReference phSessionHandle);

    /**
     * 关闭会话
     *
     * @param hSessionHandle 与密码设备已建立的会话句柄
     * @return 0 成功;   非0 失败,返回错误代码
     */
    int SDF_CloseSession(Pointer hSessionHandle);

    /**
     * 获取设备信息
     *
     * @param hSessionHandle 与设备建立的会话句柄
     * @param pstDeviceInfo  设备能力描述信息，内容及格式见设备信息定义
     * @return 0 成功;   非0 失败,返回错误代码
     */
    int SDF_GetDeviceInfo(Pointer hSessionHandle, byte[] pstDeviceInfo);

    /**
     * 产生随机数
     *
     * @param hSessionHandle 与设备建立的会话句柄
     * @param uiLength       欲获取的随机数长度
     * @param pucRandom      缓冲区指针，用于存放获取的随机数
     * @return 0 成功;   非0 失败,返回错误代码
     */
    int SDF_GenerateRandom(Pointer hSessionHandle, int uiLength, byte[] pucRandom);


}
